"""This module contains the DPF plotter class.

Contains classes used to plot a mesh and a fields container using PyVista.
"""
import tempfile
import os
import sys
import numpy as np

from ansys import dpf
from ansys.dpf import core
from ansys.dpf.core.common import locations, DefinitionLabels
from ansys.dpf.core.common import shell_layers as eshell_layers
from ansys.dpf.core import errors as dpf_errors


def plot_chart(fields_container):
    """Plot the minimum/maximum result values over time.

    This is a valid method if ``time_freq_support`` contains
    several time_steps, such as in a transient analysis.

    Parameters
    ----------
    field_container : dpf.core.FieldsContainer
        Fields container that must contains a result for each
        time step of ``time_freq_support``.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.transient_therm)
    >>> t = model.results.temperature.on_all_time_freqs()
    >>> fc = t.outputs.fields_container()
    >>> plotter = dpf.plotter.plot_chart(fc)

    """
    p = Plotter(None)
    return p.plot_chart(fields_container)


class Plotter:
    """Plots fields and meshed regions in DPF-Core.

    Parameters
    ----------
    mesh : str
        Name of the mesh.

    """

    def __init__(self, mesh):
        self._mesh = mesh

    def plot_mesh(self, **kwargs):
        """Plot the mesh using PyVista.

        Parameters
        ----------
        notebook : bool, optional
            When ``None`` (default) plot a static image within an
            iPython notebook if available.  When ``False``, plot
            external to the notebook with an interactive window.  When
            ``True``, always plot within a notebook.
        **kwargs : optional
            Additional keyword arguments for the plotter. For more information,
            ee ``help(pyvista.plot)``.

        """
        kwargs.setdefault("color", "w")
        kwargs.setdefault("show_edges", True)
        return self._mesh.grid.plot(**kwargs)

    def plot_chart(self, fields_container):
        """Plot the minimum/maximum result values over time.

        This is a valid method if ``time_freq_support`` contains
        several time steps, such as in a transient analysis.

        Parameters
        ----------
        fields_container : dpf.core.FieldsContainer
            Fields container that must contain a result for each
            time step of ``time_freq_support``.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> model = dpf.Model(examples.simple_bar)
        >>> disp = model.results.displacement()
        >>> scoping = dpf.Scoping()
        >>> scoping.ids = range(1, len(model.metadata.time_freq_support.time_frequencies) + 1)
        >>> disp.inputs.time_scoping.connect(scoping)
        >>> fc = disp.outputs.fields_container()
        >>> plotter = dpf.plotter.Plotter(model.metadata.meshed_region)
        >>> pl = plotter.plot_chart(fc)

        """
        try:
            import matplotlib.pyplot as pyplot
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "To use plot_chart capabilities, please install "
                "matplotlib with :\n pip install matplotlib>=3.2"
            )
        tfq = fields_container.time_freq_support
        if len(fields_container) != len(tfq.time_frequencies):
            raise Exception(
                "Fields container must contain real fields at all time "
                "steps of the time_freq_support."
            )
        time_field = tfq.time_frequencies
        normOp = dpf.core.Operator("norm_fc")
        minmaxOp = dpf.core.Operator("min_max_fc")
        normOp.inputs.fields_container.connect(fields_container)
        minmaxOp.inputs.connect(normOp.outputs)
        fieldMin = minmaxOp.outputs.field_min()
        fieldMax = minmaxOp.outputs.field_max()
        pyplot.plot(time_field.data, fieldMax.data, "r", label="Maximum")
        pyplot.plot(time_field.data, fieldMin.data, "b", label="Minimum")
        unit = tfq.time_frequencies.unit
        if unit == "Hz":
            pyplot.xlabel("frequencies (Hz)")
        elif unit == "s":
            pyplot.xlabel("time (s)")
        elif unit is not None:
            pyplot.xlabel(unit)
        substr = fields_container[0].name.split("_")
        pyplot.ylabel(substr[0] + fieldMin.unit)
        pyplot.title(substr[0] + ": min/max values over time")
        return pyplot.legend()

    def plot_contour(
        self,
        field_or_fields_container,
        notebook=None,
        shell_layers=None,
        off_screen=None,
        show_axes=True,
        **kwargs
    ):
        """Plot the contour result on its mesh support.

        You cannot plot a fields container containing results at several
        time steps.

        Parameters
        ----------
        field_or_fields_container : dpf.core.Field or dpf.core.FieldsContainer
            Field or field container that contains the result to plot.
        notebook : bool, optional
            Whether to plot a static image within an iPython notebook
            if available. The default is `None`, in which case an attempt is
            made to plot a static imaage within an iPython notebook. When ``False``,
            a plot external to the notebook is generated with an interactive window.
            When ``True``, a plot is always generated within a notebook.
        shell_layers : core.shell_layers, optional
            Enum used to set the shell layers if the model to plot
            contains shell elements.
        off_screen : bool, optional
            Whether to render off screen, which is useful for automated
            screenshots. The default is ``None``.
        show_axes : bool, optional
            Whether to show a VTK axes widget. The default is ``True``.
        **kwargs : optional
            Additional keyword arguments for the plotter. For more information,
            see ``help(pyvista.plot)``.
        """
        if not sys.warnoptions:
            import warnings

            warnings.simplefilter("ignore")

        if isinstance(
            field_or_fields_container, (dpf.core.Field, dpf.core.FieldsContainer)
        ):
            fields_container = None
            if isinstance(field_or_fields_container, dpf.core.Field):
                fields_container = dpf.core.FieldsContainer(
                    server=field_or_fields_container._server
                )
                fields_container.add_label(DefinitionLabels.time)
                fields_container.add_field(
                    {DefinitionLabels.time: 1}, field_or_fields_container
                )
            elif isinstance(field_or_fields_container, dpf.core.FieldsContainer):
                fields_container = field_or_fields_container
        else:
            raise TypeError("Only field or fields_container can be plotted.")

        # pre-loop to check if the there are several time steps
        labels = fields_container.get_label_space(0)
        if DefinitionLabels.complex in labels.keys():
            raise dpf_errors.ComplexPlottingError
        if DefinitionLabels.time in labels.keys():
            first_time = labels[DefinitionLabels.time]
            for i in range(1, len(fields_container)):
                label = fields_container.get_label_space(i)
                if label[DefinitionLabels.time] != first_time:
                    raise dpf_errors.FieldContainerPlottingError

        mesh = self._mesh

        # get mesh scoping
        location = None
        component_count = None
        name = None

        # pre-loop to get location and component count
        for field in fields_container:
            if len(field.data) != 0:
                location = field.location
                component_count = field.component_count
                name = field.name.split("_")[0]
                break

        if location == locations.nodal:
            mesh_location = mesh.nodes
        elif location == locations.elemental:
            mesh_location = mesh.elements
        else:
            raise ValueError(
                "Only elemental or nodal location are supported for plotting."
            )

        # pre-loop: check if shell layers for each field, if yes, set the shell layers
        changeOp = core.Operator("change_shellLayers")
        for field in fields_container:
            shell_layer_check = field.shell_layers
            if shell_layer_check in [
                eshell_layers.topbottom,
                eshell_layers.topbottommid,
            ]:
                changeOp.inputs.fields_container.connect(fields_container)
                sl = eshell_layers.top
                if shell_layers is not None:
                    if not isinstance(shell_layers, eshell_layers):
                        raise TypeError(
                            "shell_layer attribute must be a core.shell_layers instance."
                        )
                    sl = shell_layers
                changeOp.inputs.e_shell_layer.connect(sl.value)  # top layers taken
                fields_container = changeOp.outputs.fields_container()
                break

        # Merge field data into a single array
        if component_count > 1:
            overall_data = np.full((len(mesh_location), component_count), np.nan)
        else:
            overall_data = np.full(len(mesh_location), np.nan)

        for field in fields_container:
            ind, mask = mesh_location.map_scoping(field.scoping)
            overall_data[ind] = field.data[mask]

        # create the plotter and add the meshes
        background = kwargs.pop("background", None)

        try:
            import pyvista as pv
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "To use plotting capabilities, please install pyvista "
                "with :\n pip install pyvista>=0.24.0"
            )
        plotter = pv.Plotter(notebook=notebook, off_screen=off_screen)

        # add meshes
        kwargs.setdefault("show_edges", True)
        kwargs.setdefault("nan_color", "grey")
        kwargs.setdefault("stitle", name)
        plotter.add_mesh(mesh.grid, scalars=overall_data, **kwargs)

        if background is not None:
            plotter.set_background(background)

        # show result
        if show_axes:
            plotter.add_axes()
        return plotter.show()

    def _plot_contour_using_vtk_file(self, fields_container, notebook=None):
        """Plot the contour result on its mesh support.

        The resulting figure depends on the support, which can be a meshed region
        or a time freq support. If a transient analysis, the last result is plotted.

        This method is private.  DPF publishes a VTK file and displays
        this file using PyVista.
        """
        try:
            import pyvista as pv
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "To use plotting capabilities, please install pyvista "
                "with :\n pip install pyvista>=0.24.0"
            )

        plotter = pv.Plotter(notebook=notebook)
        # mesh_provider = Operator("MeshProvider")
        # mesh_provider.inputs.data_sources.connect(self._evaluator._model.metadata.data_sources)

        # create a temporary file at the default temp directory
        path = os.path.join(tempfile.gettempdir(), "dpf_temp_hokflb2j9s.vtk")

        vtk_export = dpf.core.Operator("vtk_export")
        vtk_export.inputs.mesh.connect(self._mesh)
        vtk_export.inputs.fields1.connect(fields_container)
        vtk_export.inputs.file_path.connect(path)
        vtk_export.run()
        grid = pv.read(path)

        if os.path.exists(path):
            os.remove(path)

        names = grid.array_names
        field_name = fields_container[0].name
        for n in names:  # get new name (for example if time_steps)
            if field_name in n:
                field_name = n  # default: will plot the last time_step
        val = grid.get_array(field_name)
        plotter.add_mesh(grid, scalars=val, stitle=field_name, show_edges=True)
        plotter.add_axes()
        plotter.show()



"""Download example datasets from https://github.com/pyansys/example-data"""
import shutil
import os
import urllib.request

from ansys.dpf.core import EXAMPLES_PATH

EXAMPLE_REPO = "https://github.com/pyansys/example-data/raw/master/result_files/"


def delete_downloads():
    """Delete all downloaded examples to free space or update the files"""
    shutil.rmtree(EXAMPLES_PATH)
    os.makedirs(EXAMPLES_PATH)


def _get_file_url(directory, filename):
    return EXAMPLE_REPO + "/".join([directory, filename])


def _retrieve_file(url, filename, directory):
    """Download a file from a url"""
    # First check if file has already been downloaded
    local_path = os.path.join(EXAMPLES_PATH, directory, os.path.basename(filename))
    local_path_no_zip = local_path.replace(".zip", "")
    if os.path.isfile(local_path_no_zip) or os.path.isdir(local_path_no_zip):
        return local_path_no_zip

    # grab the correct url retriever
    urlretrieve = urllib.request.urlretrieve

    dirpath = os.path.dirname(local_path)
    if not os.path.isdir(dirpath):
        os.mkdir(dirpath)

    # Perform download
    _, resp = urlretrieve(url, local_path)
    return local_path


def _download_file(directory, filename):
    url = _get_file_url(directory, filename)
    local_path = _retrieve_file(url, filename, directory)

    if os.environ.get("DPF_DOCKER", False):  # pragma: no cover
        # override path if running on docker as path must be relative
        # to docker mount
        #
        # Assumes the following mapping in docker
        # DWN_CSH=/tmp/dpf_cache
        # -v $DWN_CSH:/dpf/_cache
        local_path = os.path.join("/dpf/_cache", directory, filename)
    return local_path


###############################################################################
# front-facing functions


def download_transient_result() -> str:
    """Download an example transient result file and return the download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.transient_result
    >>> path
    'C:/Users/user/AppData/local/temp/transient.rst'

    """
    return _download_file("transient", "transient.rst")


def download_all_kinds_of_complexity() -> str:
    """Download an example static result and return the download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_all_kinds_of_complexity
    >>> path
    'C:/Users/user/AppData/local/temp/allKindOfComplexity.rst'

    """
    return _download_file("testing", "allKindOfComplexity.rst")


def download_all_kinds_of_complexity_modal() -> str:
    """Download an example result file from a static modal analysis and
    return the download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_all_kinds_of_complexity_modal()
    >>> path
    'C:/Users/user/AppData/local/temp/modal_allKindOfComplexity.rst'

    """
    return _download_file("testing", "modal_allKindOfComplexity.rst")


def download_pontoon() -> str:
    """Download an example result file from a static modal analsys and
    return the download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_pontoon()
    >>> path
    'C:/Users/user/AppData/local/temp/pontoon.rst'

    """
    return _download_file("docs", "pontoon.rst")


def download_multi_harmonic_result() -> str:
    """Download an example multi-harmonic result file and return the
    download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_multi_harmonic_result()
    >>> path
    'C:/Users/user/AppData/local/temp/file_harmonic_5rpms.rst'
    """
    return _download_file("harmonic", "file_harmonic_5rpms.rst")


def download_multi_stage_cyclic_result() -> str:
    """Download an example multi stage result file and return the
    download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_multi_stage_cyclic_result()
    >>> path
    'C:/Users/user/AppData/local/temp/multistage.rst'

    """
    return _download_file("multistage", "multistage.rst")


def download_sub_file() -> str:
    """Download an example .sub result file containing matrices and return the
    download path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_sub_file()
    >>> path
    'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\sub\\cp56.sub'

    """
    return _download_file("sub", "cp56.sub")


def download_msup_files_to_dict() -> dict:
    """Download all the files necessary for a msup expansion and return the
    download paths into a dictionnary extension->path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_msup_files_to_dict()
    >>> paths
    {'rfrq': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.rfrq',
     'mode': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.mode',
     'rst': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.rst'} # noqa: E501

    """
    return {
        "rfrq": _download_file("msup", "file.rfrq"),
        "mode": _download_file("msup", "file.mode"),
        "rst": _download_file("msup", "file.rst"),
    }


def download_distributed_files() -> dict:
    """Download distributed rst files and return the
    download paths into a dictionnary domain id->path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    dict[int:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_distributed_files()
    >>> paths
    {0: 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\distributed\\file0.rst',
     1: 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\distributed\\file1.rst'} # noqa: E501

    """
    return {
        0: _download_file("distributed", "file0.rst"),
        1: _download_file("distributed", "file1.rst"),
    }


def download_fluent_files() -> dict:
    """Download the cas and dat file of a fluent analysis and return the
    download paths into a dictionnary extension->path.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_files()
    >>> paths
    {'cas': 'C:\\Users\\cbellot\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent\\FFF.cas.h5',
     'dat': 'C:\\Users\\cbellot\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent\\FFF.dat.h5'} # noqa: E501

    """
    return {
        "cas": _download_file("fluent", "FFF.cas.h5"),
        "dat": _download_file("fluent", "FFF.dat.h5"),
    }


def download_extrapolation_3d_result() -> dict:
    """Download example static results of reference and integrated points
    for extrapolation of 3d-element and return return the dictionary of 2 download paths.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    dict
        containing path to the example file of ref and path to the example
        file of integrated points.

    Examples
    --------
    Download 2 example result files and return the dictionary containing 2 files

    >>> from ansys.dpf.core import examples
    >>> dict = examples.download_extrapolation_ref_result
    >>> dict
    {
        'file_ref': 'C:/Users/user/AppData/local/temp/file_ref.rst',
        'file_integrated': 'C:/Users/user/AppData/local/temp/file.rst'
    }

    """
    dict = {
        "file_ref": _download_file("extrapolate", "file_ref.rst"),
        "file_integrated": _download_file("extrapolate", "file.rst"),
    }

    return dict


def download_extrapolation_2d_result() -> dict:
    """Download example static results of reference and integrated points
    for extrapolation of 2d-element and return the dictionary of 2 download paths.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Returns
    -------
    dict
        Contains path to the example file of ref and path to the example
        file of integrated points.

     Examples
    --------
    Download 2 example result files and return the dictionary containing 2 files

    >>> from ansys.dpf.core import examples
    >>> dict = examples.download_extrapolation_ref_result
    >>> dict
    {
        'file_ref': 'C:/Users/user/AppData/local/temp/extrapolate_2d_ref.rst',
        'file_integrated': 'C:/Users/user/AppData/local/temp/extrapolate_2d.rst'
    }

    """
    dict = {
        "file_ref": _download_file("extrapolate", "extrapolate_2d_ref.rst"),
        "file_integrated": _download_file("extrapolate", "extrapolate_2d.rst"),
    }

    return dict



"""
splitted_on_property_type
=========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "scoping" category
"""

class splitted_on_property_type(Operator):
    """Splits a given scoping or the mesh scoping (nodal or elemental) on given properties (elshape and/or material) and returns a scopings container with those splitted scopings.

      available inputs:
        - mesh_scoping (Scoping) (optional)
        - mesh (MeshedRegion)
        - requested_location (str)
        - label1 (str) (optional)
        - label2 (str) (optional)

      available outputs:
        - mesh_scoping (ScopingsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.scoping.splitted_on_property_type()

      >>> # Make input connections
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_label1 = str()
      >>> op.inputs.label1.connect(my_label1)
      >>> my_label2 = str()
      >>> op.inputs.label2.connect(my_label2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.scoping.splitted_on_property_type(mesh_scoping=my_mesh_scoping,mesh=my_mesh,requested_location=my_requested_location,label1=my_label1,label2=my_label2)

      >>> # Get output data
      >>> result_mesh_scoping = op.outputs.mesh_scoping()"""
    def __init__(self, mesh_scoping=None, mesh=None, requested_location=None, label1=None, label2=None, config=None, server=None):
        super().__init__(name="scoping::by_property", config = config, server = server)
        self._inputs = InputsSplittedOnPropertyType(self)
        self._outputs = OutputsSplittedOnPropertyType(self)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)
        if label1 !=None:
            self.inputs.label1.connect(label1)
        if label2 !=None:
            self.inputs.label2.connect(label2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Splits a given scoping or the mesh scoping (nodal or elemental) on given properties (elshape and/or material) and returns a scopings container with those splitted scopings.""",
                             map_input_pin_spec={
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scoping"], optional=True, document="""Scoping"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document="""mesh region"""), 
                                 9 : PinSpecification(name = "requested_location", type_names=["string"], optional=False, document="""location (default is elemental)"""), 
                                 13 : PinSpecification(name = "label", type_names=["string"], optional=True, document="""properties to apply the filtering 'mat' and/or 'elshape' (default is 'elshape)"""), 
                                 14 : PinSpecification(name = "label", type_names=["string"], optional=True, document="""properties to apply the filtering 'mat' and/or 'elshape' (default is 'elshape)""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "mesh_scoping", type_names=["scopings_container"], optional=False, document="""Scoping""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "scoping::by_property")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplittedOnPropertyType 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSplittedOnPropertyType 
        """
        return super().outputs


#internal name: scoping::by_property
#scripting name: splitted_on_property_type
class InputsSplittedOnPropertyType(_Inputs):
    """Intermediate class used to connect user inputs to splitted_on_property_type operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.splitted_on_property_type()
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_label1 = str()
      >>> op.inputs.label1.connect(my_label1)
      >>> my_label2 = str()
      >>> op.inputs.label2.connect(my_label2)
    """
    def __init__(self, op: Operator):
        super().__init__(splitted_on_property_type._spec().inputs, op)
        self._mesh_scoping = Input(splitted_on_property_type._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(splitted_on_property_type._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)
        self._requested_location = Input(splitted_on_property_type._spec().input_pin(9), 9, op, -1) 
        self._inputs.append(self._requested_location)
        self._label1 = Input(splitted_on_property_type._spec().input_pin(13), 13, op, 0) 
        self._inputs.append(self._label1)
        self._label2 = Input(splitted_on_property_type._spec().input_pin(14), 14, op, 1) 
        self._inputs.append(self._label2)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: Scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: mesh region

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        - pindoc: location (default is elemental)

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

    @property
    def label1(self):
        """Allows to connect label1 input to the operator

        - pindoc: properties to apply the filtering 'mat' and/or 'elshape' (default is 'elshape)

        Parameters
        ----------
        my_label1 : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> op.inputs.label1.connect(my_label1)
        >>> #or
        >>> op.inputs.label1(my_label1)

        """
        return self._label1

    @property
    def label2(self):
        """Allows to connect label2 input to the operator

        - pindoc: properties to apply the filtering 'mat' and/or 'elshape' (default is 'elshape)

        Parameters
        ----------
        my_label2 : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> op.inputs.label2.connect(my_label2)
        >>> #or
        >>> op.inputs.label2(my_label2)

        """
        return self._label2

class OutputsSplittedOnPropertyType(_Outputs):
    """Intermediate class used to get outputs from splitted_on_property_type operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.splitted_on_property_type()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """
    def __init__(self, op: Operator):
        super().__init__(splitted_on_property_type._spec().outputs, op)
        self._mesh_scoping = Output(splitted_on_property_type._spec().output_pin(0), 0, op) 
        self._outputs.append(self._mesh_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator


        - pindoc: Scoping

        Returns
        ----------
        my_mesh_scoping : ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.splitted_on_property_type()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping() 
        """
        return self._mesh_scoping




"""
rescope
=======
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "scoping" category
"""

class rescope(Operator):
    """Rescope a field on the given scoping. If an id does not exists in the original field, default value (in 2) is used if defined.

      available inputs:
        - fields (FieldsContainer, Field)
        - mesh_scoping (Scoping, list)
        - default_value (float, list)

      available outputs:
        - fields (FieldsContainer ,Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.scoping.rescope()

      >>> # Make input connections
      >>> my_fields = dpf.FieldsContainer()
      >>> op.inputs.fields.connect(my_fields)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_default_value = float()
      >>> op.inputs.default_value.connect(my_default_value)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.scoping.rescope(fields=my_fields,mesh_scoping=my_mesh_scoping,default_value=my_default_value)

      >>> # Get output data
      >>> result_fields = op.outputs.fields()"""
    def __init__(self, fields=None, mesh_scoping=None, default_value=None, config=None, server=None):
        super().__init__(name="Rescope", config = config, server = server)
        self._inputs = InputsRescope(self)
        self._outputs = OutputsRescope(self)
        if fields !=None:
            self.inputs.fields.connect(fields)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if default_value !=None:
            self.inputs.default_value.connect(default_value)

    @staticmethod
    def _spec():
        spec = Specification(description="""Rescope a field on the given scoping. If an id does not exists in the original field, default value (in 2) is used if defined.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields", type_names=["fields_container","field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scoping","vector<int32>"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "default_value", type_names=["double","vector<double>"], optional=False, document="""if a the pin 2 is used, the ids not found in the fields are added with this default value""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields", type_names=["fields_container","field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "Rescope")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRescope 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsRescope 
        """
        return super().outputs


#internal name: Rescope
#scripting name: rescope
class InputsRescope(_Inputs):
    """Intermediate class used to connect user inputs to rescope operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.rescope()
      >>> my_fields = dpf.FieldsContainer()
      >>> op.inputs.fields.connect(my_fields)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_default_value = float()
      >>> op.inputs.default_value.connect(my_default_value)
    """
    def __init__(self, op: Operator):
        super().__init__(rescope._spec().inputs, op)
        self._fields = Input(rescope._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields)
        self._mesh_scoping = Input(rescope._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._default_value = Input(rescope._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._default_value)

    @property
    def fields(self):
        """Allows to connect fields input to the operator

        Parameters
        ----------
        my_fields : FieldsContainer, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.rescope()
        >>> op.inputs.fields.connect(my_fields)
        >>> #or
        >>> op.inputs.fields(my_fields)

        """
        return self._fields

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        Parameters
        ----------
        my_mesh_scoping : Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.rescope()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def default_value(self):
        """Allows to connect default_value input to the operator

        - pindoc: if a the pin 2 is used, the ids not found in the fields are added with this default value

        Parameters
        ----------
        my_default_value : float, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.scoping.rescope()
        >>> op.inputs.default_value.connect(my_default_value)
        >>> #or
        >>> op.inputs.default_value(my_default_value)

        """
        return self._default_value

class OutputsRescope(_Outputs):
    """Intermediate class used to get outputs from rescope operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.scoping.rescope()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields = op.outputs.fields()
    """
    def __init__(self, op: Operator):
        super().__init__(rescope._spec().outputs, op)
        self.fields_as_fields_container = Output( _modify_output_spec_with_one_type(rescope._spec().output_pin(0), "fields_container"), 0, op) 
        self._outputs.append(self.fields_as_fields_container)
        self.fields_as_field = Output( _modify_output_spec_with_one_type(rescope._spec().output_pin(0), "field"), 0, op) 
        self._outputs.append(self.fields_as_field)




"""
min_max_inc
===========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class min_max_inc(Operator):
    """Compute the component-wise minimum (out 0) and maximum (out 1) over coming fields.

      available inputs:
        - field (Field)
        - domain_id (int) (optional)

      available outputs:
        - field_min (Field)
        - field_max (Field)
        - domain_ids_min (Scoping)
        - domain_ids_max (Scoping)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.min_max_inc()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_domain_id = int()
      >>> op.inputs.domain_id.connect(my_domain_id)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.min_max_inc(field=my_field,domain_id=my_domain_id)

      >>> # Get output data
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()
      >>> result_domain_ids_min = op.outputs.domain_ids_min()
      >>> result_domain_ids_max = op.outputs.domain_ids_max()"""
    def __init__(self, field=None, domain_id=None, config=None, server=None):
        super().__init__(name="min_max_inc", config = config, server = server)
        self._inputs = InputsMinMaxInc(self)
        self._outputs = OutputsMinMaxInc(self)
        if field !=None:
            self.inputs.field.connect(field)
        if domain_id !=None:
            self.inputs.domain_id.connect(domain_id)

    @staticmethod
    def _spec():
        spec = Specification(description="""Compute the component-wise minimum (out 0) and maximum (out 1) over coming fields.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document=""""""), 
                                 17 : PinSpecification(name = "domain_id", type_names=["int32"], optional=True, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field_min", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "field_max", type_names=["field"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "domain_ids_min", type_names=["scoping"], optional=False, document=""""""), 
                                 3 : PinSpecification(name = "domain_ids_max", type_names=["scoping"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "min_max_inc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinMaxInc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMinMaxInc 
        """
        return super().outputs


#internal name: min_max_inc
#scripting name: min_max_inc
class InputsMinMaxInc(_Inputs):
    """Intermediate class used to connect user inputs to min_max_inc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max_inc()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_domain_id = int()
      >>> op.inputs.domain_id.connect(my_domain_id)
    """
    def __init__(self, op: Operator):
        super().__init__(min_max_inc._spec().inputs, op)
        self._field = Input(min_max_inc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._domain_id = Input(min_max_inc._spec().input_pin(17), 17, op, -1) 
        self._inputs.append(self._domain_id)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def domain_id(self):
        """Allows to connect domain_id input to the operator

        Parameters
        ----------
        my_domain_id : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> op.inputs.domain_id.connect(my_domain_id)
        >>> #or
        >>> op.inputs.domain_id(my_domain_id)

        """
        return self._domain_id

class OutputsMinMaxInc(_Outputs):
    """Intermediate class used to get outputs from min_max_inc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max_inc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()
      >>> result_domain_ids_min = op.outputs.domain_ids_min()
      >>> result_domain_ids_max = op.outputs.domain_ids_max()
    """
    def __init__(self, op: Operator):
        super().__init__(min_max_inc._spec().outputs, op)
        self._field_min = Output(min_max_inc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field_min)
        self._field_max = Output(min_max_inc._spec().output_pin(1), 1, op) 
        self._outputs.append(self._field_max)
        self._domain_ids_min = Output(min_max_inc._spec().output_pin(2), 2, op) 
        self._outputs.append(self._domain_ids_min)
        self._domain_ids_max = Output(min_max_inc._spec().output_pin(3), 3, op) 
        self._outputs.append(self._domain_ids_max)

    @property
    def field_min(self):
        """Allows to get field_min output of the operator


        Returns
        ----------
        my_field_min : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_min = op.outputs.field_min() 
        """
        return self._field_min

    @property
    def field_max(self):
        """Allows to get field_max output of the operator


        Returns
        ----------
        my_field_max : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_max = op.outputs.field_max() 
        """
        return self._field_max

    @property
    def domain_ids_min(self):
        """Allows to get domain_ids_min output of the operator


        Returns
        ----------
        my_domain_ids_min : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_domain_ids_min = op.outputs.domain_ids_min() 
        """
        return self._domain_ids_min

    @property
    def domain_ids_max(self):
        """Allows to get domain_ids_max output of the operator


        Returns
        ----------
        my_domain_ids_max : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_inc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_domain_ids_max = op.outputs.domain_ids_max() 
        """
        return self._domain_ids_max




"""
remove_rigid_body_motion
========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class remove_rigid_body_motion(Operator):
    """Removes rigid body mode from a total displacement field by minimization. Use a reference point in order to substract its displacement to the result displacement field.

      available inputs:
        - field (Field, FieldsContainer)
        - reference_node_id (int) (optional)
        - mesh (MeshedRegion) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.remove_rigid_body_motion()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_reference_node_id = int()
      >>> op.inputs.reference_node_id.connect(my_reference_node_id)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.remove_rigid_body_motion(field=my_field,reference_node_id=my_reference_node_id,mesh=my_mesh)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, field=None, reference_node_id=None, mesh=None, config=None, server=None):
        super().__init__(name="ExtractRigidBodyMotion", config = config, server = server)
        self._inputs = InputsRemoveRigidBodyMotion(self)
        self._outputs = OutputsRemoveRigidBodyMotion(self)
        if field !=None:
            self.inputs.field.connect(field)
        if reference_node_id !=None:
            self.inputs.reference_node_id.connect(reference_node_id)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Removes rigid body mode from a total displacement field by minimization. Use a reference point in order to substract its displacement to the result displacement field.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 1 : PinSpecification(name = "reference_node_id", type_names=["int32"], optional=True, document="""Id of the reference entity (node)."""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""default is the mesh in the support""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "ExtractRigidBodyMotion")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRemoveRigidBodyMotion 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsRemoveRigidBodyMotion 
        """
        return super().outputs


#internal name: ExtractRigidBodyMotion
#scripting name: remove_rigid_body_motion
class InputsRemoveRigidBodyMotion(_Inputs):
    """Intermediate class used to connect user inputs to remove_rigid_body_motion operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.remove_rigid_body_motion()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_reference_node_id = int()
      >>> op.inputs.reference_node_id.connect(my_reference_node_id)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(remove_rigid_body_motion._spec().inputs, op)
        self._field = Input(remove_rigid_body_motion._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._reference_node_id = Input(remove_rigid_body_motion._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._reference_node_id)
        self._mesh = Input(remove_rigid_body_motion._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def field(self):
        """Allows to connect field input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_field : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.remove_rigid_body_motion()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def reference_node_id(self):
        """Allows to connect reference_node_id input to the operator

        - pindoc: Id of the reference entity (node).

        Parameters
        ----------
        my_reference_node_id : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.remove_rigid_body_motion()
        >>> op.inputs.reference_node_id.connect(my_reference_node_id)
        >>> #or
        >>> op.inputs.reference_node_id(my_reference_node_id)

        """
        return self._reference_node_id

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: default is the mesh in the support

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.remove_rigid_body_motion()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsRemoveRigidBodyMotion(_Outputs):
    """Intermediate class used to get outputs from remove_rigid_body_motion operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.remove_rigid_body_motion()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(remove_rigid_body_motion._spec().outputs, op)
        self._field = Output(remove_rigid_body_motion._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.remove_rigid_body_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
add_rigid_body_motion
=====================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "result" category
"""

class add_rigid_body_motion(Operator):
    """Adds a given rigid translation, center and rotation from a displacement field. The rotation is given in terms of rotations angles. Note that the displacement field has to be in the global coordinate sytem

      available inputs:
        - displacement_field (Field)
        - translation_field (Field)
        - rotation_field (Field)
        - center_field (Field)
        - mesh (MeshedRegion) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.add_rigid_body_motion()

      >>> # Make input connections
      >>> my_displacement_field = dpf.Field()
      >>> op.inputs.displacement_field.connect(my_displacement_field)
      >>> my_translation_field = dpf.Field()
      >>> op.inputs.translation_field.connect(my_translation_field)
      >>> my_rotation_field = dpf.Field()
      >>> op.inputs.rotation_field.connect(my_rotation_field)
      >>> my_center_field = dpf.Field()
      >>> op.inputs.center_field.connect(my_center_field)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.add_rigid_body_motion(displacement_field=my_displacement_field,translation_field=my_translation_field,rotation_field=my_rotation_field,center_field=my_center_field,mesh=my_mesh)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, displacement_field=None, translation_field=None, rotation_field=None, center_field=None, mesh=None, config=None, server=None):
        super().__init__(name="RigidBodyAddition", config = config, server = server)
        self._inputs = InputsAddRigidBodyMotion(self)
        self._outputs = OutputsAddRigidBodyMotion(self)
        if displacement_field !=None:
            self.inputs.displacement_field.connect(displacement_field)
        if translation_field !=None:
            self.inputs.translation_field.connect(translation_field)
        if rotation_field !=None:
            self.inputs.rotation_field.connect(rotation_field)
        if center_field !=None:
            self.inputs.center_field.connect(center_field)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Adds a given rigid translation, center and rotation from a displacement field. The rotation is given in terms of rotations angles. Note that the displacement field has to be in the global coordinate sytem""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "displacement_field", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "translation_field", type_names=["field"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "rotation_field", type_names=["field"], optional=False, document=""""""), 
                                 3 : PinSpecification(name = "center_field", type_names=["field"], optional=False, document=""""""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""default is the mesh in the support""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "RigidBodyAddition")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsAddRigidBodyMotion 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsAddRigidBodyMotion 
        """
        return super().outputs


#internal name: RigidBodyAddition
#scripting name: add_rigid_body_motion
class InputsAddRigidBodyMotion(_Inputs):
    """Intermediate class used to connect user inputs to add_rigid_body_motion operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.add_rigid_body_motion()
      >>> my_displacement_field = dpf.Field()
      >>> op.inputs.displacement_field.connect(my_displacement_field)
      >>> my_translation_field = dpf.Field()
      >>> op.inputs.translation_field.connect(my_translation_field)
      >>> my_rotation_field = dpf.Field()
      >>> op.inputs.rotation_field.connect(my_rotation_field)
      >>> my_center_field = dpf.Field()
      >>> op.inputs.center_field.connect(my_center_field)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(add_rigid_body_motion._spec().inputs, op)
        self._displacement_field = Input(add_rigid_body_motion._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._displacement_field)
        self._translation_field = Input(add_rigid_body_motion._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._translation_field)
        self._rotation_field = Input(add_rigid_body_motion._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._rotation_field)
        self._center_field = Input(add_rigid_body_motion._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._center_field)
        self._mesh = Input(add_rigid_body_motion._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def displacement_field(self):
        """Allows to connect displacement_field input to the operator

        Parameters
        ----------
        my_displacement_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> op.inputs.displacement_field.connect(my_displacement_field)
        >>> #or
        >>> op.inputs.displacement_field(my_displacement_field)

        """
        return self._displacement_field

    @property
    def translation_field(self):
        """Allows to connect translation_field input to the operator

        Parameters
        ----------
        my_translation_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> op.inputs.translation_field.connect(my_translation_field)
        >>> #or
        >>> op.inputs.translation_field(my_translation_field)

        """
        return self._translation_field

    @property
    def rotation_field(self):
        """Allows to connect rotation_field input to the operator

        Parameters
        ----------
        my_rotation_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> op.inputs.rotation_field.connect(my_rotation_field)
        >>> #or
        >>> op.inputs.rotation_field(my_rotation_field)

        """
        return self._rotation_field

    @property
    def center_field(self):
        """Allows to connect center_field input to the operator

        Parameters
        ----------
        my_center_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> op.inputs.center_field.connect(my_center_field)
        >>> #or
        >>> op.inputs.center_field(my_center_field)

        """
        return self._center_field

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: default is the mesh in the support

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsAddRigidBodyMotion(_Outputs):
    """Intermediate class used to get outputs from add_rigid_body_motion operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.add_rigid_body_motion()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(add_rigid_body_motion._spec().outputs, op)
        self._field = Output(add_rigid_body_motion._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.add_rigid_body_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
phase_of_max
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class phase_of_max(Operator):
    """Evaluates phase of maximum.

      available inputs:
        - real_field (Field)
        - imaginary_field (Field)
        - abs_value (bool) (optional)
        - phase_increment (float)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.phase_of_max()

      >>> # Make input connections
      >>> my_real_field = dpf.Field()
      >>> op.inputs.real_field.connect(my_real_field)
      >>> my_imaginary_field = dpf.Field()
      >>> op.inputs.imaginary_field.connect(my_imaginary_field)
      >>> my_abs_value = bool()
      >>> op.inputs.abs_value.connect(my_abs_value)
      >>> my_phase_increment = float()
      >>> op.inputs.phase_increment.connect(my_phase_increment)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.phase_of_max(real_field=my_real_field,imaginary_field=my_imaginary_field,abs_value=my_abs_value,phase_increment=my_phase_increment)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, real_field=None, imaginary_field=None, abs_value=None, phase_increment=None, config=None, server=None):
        super().__init__(name="phase_of_max", config = config, server = server)
        self._inputs = InputsPhaseOfMax(self)
        self._outputs = OutputsPhaseOfMax(self)
        if real_field !=None:
            self.inputs.real_field.connect(real_field)
        if imaginary_field !=None:
            self.inputs.imaginary_field.connect(imaginary_field)
        if abs_value !=None:
            self.inputs.abs_value.connect(abs_value)
        if phase_increment !=None:
            self.inputs.phase_increment.connect(phase_increment)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluates phase of maximum.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "real_field", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "imaginary_field", type_names=["field"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "abs_value", type_names=["bool"], optional=True, document="""Should use absolute value."""), 
                                 3 : PinSpecification(name = "phase_increment", type_names=["double"], optional=False, document="""Phase increment.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "phase_of_max")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsPhaseOfMax 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsPhaseOfMax 
        """
        return super().outputs


#internal name: phase_of_max
#scripting name: phase_of_max
class InputsPhaseOfMax(_Inputs):
    """Intermediate class used to connect user inputs to phase_of_max operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.phase_of_max()
      >>> my_real_field = dpf.Field()
      >>> op.inputs.real_field.connect(my_real_field)
      >>> my_imaginary_field = dpf.Field()
      >>> op.inputs.imaginary_field.connect(my_imaginary_field)
      >>> my_abs_value = bool()
      >>> op.inputs.abs_value.connect(my_abs_value)
      >>> my_phase_increment = float()
      >>> op.inputs.phase_increment.connect(my_phase_increment)
    """
    def __init__(self, op: Operator):
        super().__init__(phase_of_max._spec().inputs, op)
        self._real_field = Input(phase_of_max._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._real_field)
        self._imaginary_field = Input(phase_of_max._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._imaginary_field)
        self._abs_value = Input(phase_of_max._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._abs_value)
        self._phase_increment = Input(phase_of_max._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._phase_increment)

    @property
    def real_field(self):
        """Allows to connect real_field input to the operator

        Parameters
        ----------
        my_real_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.phase_of_max()
        >>> op.inputs.real_field.connect(my_real_field)
        >>> #or
        >>> op.inputs.real_field(my_real_field)

        """
        return self._real_field

    @property
    def imaginary_field(self):
        """Allows to connect imaginary_field input to the operator

        Parameters
        ----------
        my_imaginary_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.phase_of_max()
        >>> op.inputs.imaginary_field.connect(my_imaginary_field)
        >>> #or
        >>> op.inputs.imaginary_field(my_imaginary_field)

        """
        return self._imaginary_field

    @property
    def abs_value(self):
        """Allows to connect abs_value input to the operator

        - pindoc: Should use absolute value.

        Parameters
        ----------
        my_abs_value : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.phase_of_max()
        >>> op.inputs.abs_value.connect(my_abs_value)
        >>> #or
        >>> op.inputs.abs_value(my_abs_value)

        """
        return self._abs_value

    @property
    def phase_increment(self):
        """Allows to connect phase_increment input to the operator

        - pindoc: Phase increment.

        Parameters
        ----------
        my_phase_increment : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.phase_of_max()
        >>> op.inputs.phase_increment.connect(my_phase_increment)
        >>> #or
        >>> op.inputs.phase_increment(my_phase_increment)

        """
        return self._phase_increment

class OutputsPhaseOfMax(_Outputs):
    """Intermediate class used to get outputs from phase_of_max operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.phase_of_max()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(phase_of_max._spec().outputs, op)
        self._field = Output(phase_of_max._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.phase_of_max()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""Version for ansys-dpf-core"""
# major, minor, patch
version_info = 0, 3, 1

# Nice string for the version
__version__ = ".".join(map(str, version_info))
__ansys_version__ = "212"
min_server_version = "2.0"



"""
mechanical_csv_to_field
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "serialization" category
"""

class mechanical_csv_to_field(Operator):
    """Reads mechanical exported csv file

      available inputs:
        - unit ()
        - mesh (MeshedRegion) (optional)
        - data_sources (DataSources)
        - requested_location (str, FieldDefinition)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.mechanical_csv_to_field()

      >>> # Make input connections
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.mechanical_csv_to_field(mesh=my_mesh,data_sources=my_data_sources,requested_location=my_requested_location)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, mesh=None, data_sources=None, requested_location=None, config=None, server=None):
        super().__init__(name="mechanical_csv_to_field", config = config, server = server)
        self._inputs = InputsMechanicalCsvToField(self)
        self._outputs = OutputsMechanicalCsvToField(self)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        spec = Specification(description="""Reads mechanical exported csv file""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "unit", type_names=[], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document=""""""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document=""""""), 
                                 9 : PinSpecification(name = "requested_location", type_names=["string","field_definition"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mechanical_csv_to_field")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMechanicalCsvToField 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMechanicalCsvToField 
        """
        return super().outputs


#internal name: mechanical_csv_to_field
#scripting name: mechanical_csv_to_field
class InputsMechanicalCsvToField(_Inputs):
    """Intermediate class used to connect user inputs to mechanical_csv_to_field operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.mechanical_csv_to_field()
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
    """
    def __init__(self, op: Operator):
        super().__init__(mechanical_csv_to_field._spec().inputs, op)
        self._mesh = Input(mechanical_csv_to_field._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._data_sources = Input(mechanical_csv_to_field._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._requested_location = Input(mechanical_csv_to_field._spec().input_pin(9), 9, op, -1) 
        self._inputs.append(self._requested_location)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        Parameters
        ----------
        my_requested_location : str, FieldDefinition, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

class OutputsMechanicalCsvToField(_Outputs):
    """Intermediate class used to get outputs from mechanical_csv_to_field operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.mechanical_csv_to_field()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(mechanical_csv_to_field._spec().outputs, op)
        self._field = Output(mechanical_csv_to_field._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




# -*- coding: utf-8 -*-
"""
MeshesContainer
===============
Contains classes associated with the DPF MeshesContainer.
"""
from ansys import dpf
from ansys.dpf.core.collection import Collection
from ansys.dpf.core.common import types


class MeshesContainer(Collection):
    """Represents a meshes container, which contains meshes split on a given space.

    Parameters
    ----------
    meshes_container : ansys.grpc.dpf.collection_pb2.Collection or
                       ansys.dpf.core.MeshesContainer, optional
        Create a meshes container from a collection message or create a copy from an
        existing meshes container. The default is ``None``.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.
    """

    def __init__(self, meshes_container=None, server=None):
        """Initialize the scoping with either an optional scoping
        message or by connecting to a stub."""
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect()

        Collection.__init__(
            self, types.meshed_region, collection=meshes_container, server=self._server
        )

    def get_meshes(self, label_space):
        """Retrieve the meshes at a label space.

        Parameters
        ----------
        label_space : dict[str,int]
            Meshes corresponding to a filter (label space) in the input. For example:
            ``{"elshape":1, "body":12}``

        Returns
        -------
        meshes : list[MeshedRegion]
            Meshes corresponding to the request.
        """
        return super()._get_entries(label_space)

    def get_mesh(self, label_space_or_index):
        """Retrieve the mesh at a requested index or label space.

        Raises an exception if the request returns more than one mesh.

        Parameters
        ----------
        label_space_or_index : dict[str,int] , int
            Scoping of the requested mesh, such as ``{"time": 1, "complex": 0}``
            or the index of the mesh.

        Returns
        -------
        mesh : MeshedRegion
            Mesh corresponding to the request.
        """
        return super()._get_entry(label_space_or_index)

    def __getitem__(self, key):
        """Retrieves the mesh at a requested index.

        Parameters
        ----------
        key : int
            Index

        Returns
        -------
        mesh : MeshedRegion
            Mesh corresponding to the request.
        """
        return super().__getitem__(key)

    def add_mesh(self, label_space, mesh):
        """Add or update the scoping at a requested label space.

        Parameters
        ----------
        label_space : dict[str,int]
            Label space of the requested meshes. For example, {"elshape":1, "body":12}.

        mesh : MeshedRegion
            DPF mesh to add or update.
        """
        return super()._add_entry(label_space, mesh)

    def __str__(self):
        txt = "DPF Meshes Container with\n"
        txt += "\t%d mesh(es)\n" % len(self)
        txt += f"\tdefined on labels {self.labels} \n\n"
        return txt



"""
modal_superposition
===================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "math" category
"""

class modal_superposition(Operator):
    """Compute the solution in the time/frequency space from a modal solution by multiplying a modal basis (in 0) by the solution in this modal space (coefficients for each mode for each time/frequency) (in 1).

      available inputs:
        - modal_basis (FieldsContainer)
        - solution_in_modal_space (FieldsContainer)
        - time_scoping (Scoping, list) (optional)
        - mesh_scoping (Scoping, ScopingsContainer) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.modal_superposition()

      >>> # Make input connections
      >>> my_modal_basis = dpf.FieldsContainer()
      >>> op.inputs.modal_basis.connect(my_modal_basis)
      >>> my_solution_in_modal_space = dpf.FieldsContainer()
      >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.modal_superposition(modal_basis=my_modal_basis,solution_in_modal_space=my_solution_in_modal_space,time_scoping=my_time_scoping,mesh_scoping=my_mesh_scoping)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, modal_basis=None, solution_in_modal_space=None, time_scoping=None, mesh_scoping=None, config=None, server=None):
        super().__init__(name="expansion::modal_superposition", config = config, server = server)
        self._inputs = InputsModalSuperposition(self)
        self._outputs = OutputsModalSuperposition(self)
        if modal_basis !=None:
            self.inputs.modal_basis.connect(modal_basis)
        if solution_in_modal_space !=None:
            self.inputs.solution_in_modal_space.connect(solution_in_modal_space)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)

    @staticmethod
    def _spec():
        spec = Specification(description="""Compute the solution in the time/frequency space from a modal solution by multiplying a modal basis (in 0) by the solution in this modal space (coefficients for each mode for each time/frequency) (in 1).""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "modal_basis", type_names=["fields_container"], optional=False, document="""one field by mode with each field representing a mode shape on nodes or elements"""), 
                                 1 : PinSpecification(name = "solution_in_modal_space", type_names=["fields_container"], optional=False, document="""one field by time/frequency with each field having a ponderating coefficient for each mode of the modal_basis pin"""), 
                                 3 : PinSpecification(name = "time_scoping", type_names=["scoping","vector<int32>"], optional=True, document="""this input allows to compute the result on a subset of the time frequency domain defined in the solution_in_modal_space fields container"""), 
                                 4 : PinSpecification(name = "mesh_scoping", type_names=["scoping","scopings_container"], optional=True, document="""this input allows to compute the result on a subset of the space domain defined in the modal_basis fields container""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "expansion::modal_superposition")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsModalSuperposition 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsModalSuperposition 
        """
        return super().outputs


#internal name: expansion::modal_superposition
#scripting name: modal_superposition
class InputsModalSuperposition(_Inputs):
    """Intermediate class used to connect user inputs to modal_superposition operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.modal_superposition()
      >>> my_modal_basis = dpf.FieldsContainer()
      >>> op.inputs.modal_basis.connect(my_modal_basis)
      >>> my_solution_in_modal_space = dpf.FieldsContainer()
      >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.Scoping()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    """
    def __init__(self, op: Operator):
        super().__init__(modal_superposition._spec().inputs, op)
        self._modal_basis = Input(modal_superposition._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._modal_basis)
        self._solution_in_modal_space = Input(modal_superposition._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._solution_in_modal_space)
        self._time_scoping = Input(modal_superposition._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(modal_superposition._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._mesh_scoping)

    @property
    def modal_basis(self):
        """Allows to connect modal_basis input to the operator

        - pindoc: one field by mode with each field representing a mode shape on nodes or elements

        Parameters
        ----------
        my_modal_basis : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.modal_basis.connect(my_modal_basis)
        >>> #or
        >>> op.inputs.modal_basis(my_modal_basis)

        """
        return self._modal_basis

    @property
    def solution_in_modal_space(self):
        """Allows to connect solution_in_modal_space input to the operator

        - pindoc: one field by time/frequency with each field having a ponderating coefficient for each mode of the modal_basis pin

        Parameters
        ----------
        my_solution_in_modal_space : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
        >>> #or
        >>> op.inputs.solution_in_modal_space(my_solution_in_modal_space)

        """
        return self._solution_in_modal_space

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        - pindoc: this input allows to compute the result on a subset of the time frequency domain defined in the solution_in_modal_space fields container

        Parameters
        ----------
        my_time_scoping : Scoping, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: this input allows to compute the result on a subset of the space domain defined in the modal_basis fields container

        Parameters
        ----------
        my_mesh_scoping : Scoping, ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

class OutputsModalSuperposition(_Outputs):
    """Intermediate class used to get outputs from modal_superposition operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.modal_superposition()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(modal_superposition._spec().outputs, op)
        self._fields_container = Output(modal_superposition._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.modal_superposition()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
element_nodal_contribution
==========================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "geo" category
"""

class element_nodal_contribution(Operator):
    """Compute the fraction of volume attributed to each node of each element.

      available inputs:
        - mesh (MeshedRegion)
        - scoping (Scoping) (optional)
        - volume_fraction (bool) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.geo.element_nodal_contribution()

      >>> # Make input connections
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_volume_fraction = bool()
      >>> op.inputs.volume_fraction.connect(my_volume_fraction)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.geo.element_nodal_contribution(mesh=my_mesh,scoping=my_scoping,volume_fraction=my_volume_fraction)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, mesh=None, scoping=None, volume_fraction=None, config=None, server=None):
        super().__init__(name="element::nodal_contribution", config = config, server = server)
        self._inputs = InputsElementNodalContribution(self)
        self._outputs = OutputsElementNodalContribution(self)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if volume_fraction !=None:
            self.inputs.volume_fraction.connect(volume_fraction)

    @staticmethod
    def _spec():
        spec = Specification(description="""Compute the fraction of volume attributed to each node of each element.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document="""Integrate the input field over a specific scoping."""), 
                                 2 : PinSpecification(name = "volume_fraction", type_names=["bool"], optional=True, document="""if true, returns influence volume, if false, return influence volume fraction (i.e. integrated value of shape function for each node).""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "element::nodal_contribution")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementNodalContribution 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElementNodalContribution 
        """
        return super().outputs


#internal name: element::nodal_contribution
#scripting name: element_nodal_contribution
class InputsElementNodalContribution(_Inputs):
    """Intermediate class used to connect user inputs to element_nodal_contribution operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.element_nodal_contribution()
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_volume_fraction = bool()
      >>> op.inputs.volume_fraction.connect(my_volume_fraction)
    """
    def __init__(self, op: Operator):
        super().__init__(element_nodal_contribution._spec().inputs, op)
        self._mesh = Input(element_nodal_contribution._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._mesh)
        self._scoping = Input(element_nodal_contribution._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._scoping)
        self._volume_fraction = Input(element_nodal_contribution._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._volume_fraction)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.element_nodal_contribution()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: Integrate the input field over a specific scoping.

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.element_nodal_contribution()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def volume_fraction(self):
        """Allows to connect volume_fraction input to the operator

        - pindoc: if true, returns influence volume, if false, return influence volume fraction (i.e. integrated value of shape function for each node).

        Parameters
        ----------
        my_volume_fraction : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.element_nodal_contribution()
        >>> op.inputs.volume_fraction.connect(my_volume_fraction)
        >>> #or
        >>> op.inputs.volume_fraction(my_volume_fraction)

        """
        return self._volume_fraction

class OutputsElementNodalContribution(_Outputs):
    """Intermediate class used to get outputs from element_nodal_contribution operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.element_nodal_contribution()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(element_nodal_contribution._spec().outputs, op)
        self._field = Output(element_nodal_contribution._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.element_nodal_contribution()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
forward
=======
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "utility" category
"""

class forward(Operator):
    """Return all the inputs as outputs.

      available inputs:
        - any (Any)

      available outputs:
        - any ()

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.utility.forward()

      >>> # Make input connections
      >>> my_any = dpf.Any()
      >>> op.inputs.any.connect(my_any)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.utility.forward(any=my_any)

      >>> # Get output data
      >>> result_any = op.outputs.any()"""
    def __init__(self, any=None, config=None, server=None):
        super().__init__(name="forward", config = config, server = server)
        self._inputs = InputsForward(self)
        self._outputs = OutputsForward(self)
        if any !=None:
            self.inputs.any.connect(any)

    @staticmethod
    def _spec():
        spec = Specification(description="""Return all the inputs as outputs.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "any", type_names=["any"], optional=False, document="""any type of input""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "any", type_names=[], optional=False, document="""same types as inputs""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "forward")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsForward 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsForward 
        """
        return super().outputs


#internal name: forward
#scripting name: forward
class InputsForward(_Inputs):
    """Intermediate class used to connect user inputs to forward operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.forward()
      >>> my_any = dpf.Any()
      >>> op.inputs.any.connect(my_any)
    """
    def __init__(self, op: Operator):
        super().__init__(forward._spec().inputs, op)
        self._any = Input(forward._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._any)

    @property
    def any(self):
        """Allows to connect any input to the operator

        - pindoc: any type of input

        Parameters
        ----------
        my_any : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.forward()
        >>> op.inputs.any.connect(my_any)
        >>> #or
        >>> op.inputs.any(my_any)

        """
        return self._any

class OutputsForward(_Outputs):
    """Intermediate class used to get outputs from forward operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.forward()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(forward._spec().outputs, op)
        pass 




from .minus import minus 
from .cplx_multiply import cplx_multiply 
from .unit_convert import unit_convert 
from .minus_fc import minus_fc 
from .accumulate import accumulate 
from .unit_convert_fc import unit_convert_fc 
from .accumulate_min_over_label_fc import accumulate_min_over_label_fc 
from .add import add 
from .add_fc import add_fc 
from .sin_fc import sin_fc 
from .add_constant import add_constant 
from .invert_fc import invert_fc 
from .pow import pow 
from .add_constant_fc import add_constant_fc 
from .scale import scale 
from .pow_fc import pow_fc 
from .scale_fc import scale_fc 
from .centroid import centroid 
from .sweeping_phase import sweeping_phase 
from .centroid_fc import centroid_fc 
from .sweeping_phase_fc import sweeping_phase_fc 
from .sqr import sqr 
from .sin import sin 
from .cos import cos 
from .cos_fc import cos_fc 
from .linear_combination import linear_combination 
from .sqr_fc import sqr_fc 
from .sqrt import sqrt 
from .norm import norm 
from .sqrt_fc import sqrt_fc 
from .norm_fc import norm_fc 
from .component_wise_divide import component_wise_divide 
from .component_wise_divide_fc import component_wise_divide_fc 
from .kronecker_prod import kronecker_prod 
from .real_part import real_part 
from .conjugate import conjugate 
from .img_part import img_part 
from .amplitude import amplitude 
from .cplx_add import cplx_add 
from .cplx_dot import cplx_dot 
from .cplx_divide import cplx_divide 
from .dot import dot 
from .cplx_derive import cplx_derive 
from .polar_to_cplx import polar_to_cplx 
from .amplitude_fc import amplitude_fc 
from .scale_by_field import scale_by_field 
from .generalized_inner_product_fc import generalized_inner_product_fc 
from .phase import phase 
from .scale_by_field_fc import scale_by_field_fc 
from .phase_fc import phase_fc 
from .modulus import modulus 
from .accumulate_fc import accumulate_fc 
from .generalized_inner_product import generalized_inner_product 
from .overall_dot import overall_dot 
from .invert import invert 
from .dot_tensor import dot_tensor 
from .average_over_label_fc import average_over_label_fc 
from .accumulate_over_label_fc import accumulate_over_label_fc 
from .accumulate_level_over_label_fc import accumulate_level_over_label_fc 
from .modal_superposition import modal_superposition 
from .matrix_inverse import matrix_inverse 
from .qr_solve import qr_solve 
from .fft_eval import fft_eval 
from .fft_gradient_eval import fft_gradient_eval 
from .fft_multi_harmonic_minmax import fft_multi_harmonic_minmax 
from .svd import svd 
from .matrix_inverse import matrix_inverse 
from .qr_solve import qr_solve 
from .fft_eval import fft_eval 
from .fft_gradient_eval import fft_gradient_eval 
from .fft_multi_harmonic_minmax import fft_multi_harmonic_minmax 
from .svd import svd 
from .matrix_inverse import matrix_inverse 
from .qr_solve import qr_solve 
from .fft_eval import fft_eval 
from .fft_gradient_eval import fft_gradient_eval 
from .fft_multi_harmonic_minmax import fft_multi_harmonic_minmax 
from .svd import svd 



"""
on_coordinates
==============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "mapping" category
"""

class on_coordinates(Operator):
    """Evaluates a result on specified coordinates (interpolates results inside elements with shape functions).

      available inputs:
        - fields_container (FieldsContainer)
        - coordinates (Field, FieldsContainer)
        - create_support (bool) (optional)
        - mapping_on_scoping (bool) (optional)
        - mesh (MeshedRegion, MeshesContainer) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mapping.on_coordinates()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_coordinates = dpf.Field()
      >>> op.inputs.coordinates.connect(my_coordinates)
      >>> my_create_support = bool()
      >>> op.inputs.create_support.connect(my_create_support)
      >>> my_mapping_on_scoping = bool()
      >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mapping.on_coordinates(fields_container=my_fields_container,coordinates=my_coordinates,create_support=my_create_support,mapping_on_scoping=my_mapping_on_scoping,mesh=my_mesh)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, coordinates=None, create_support=None, mapping_on_scoping=None, mesh=None, config=None, server=None):
        super().__init__(name="mapping", config = config, server = server)
        self._inputs = InputsOnCoordinates(self)
        self._outputs = OutputsOnCoordinates(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if coordinates !=None:
            self.inputs.coordinates.connect(coordinates)
        if create_support !=None:
            self.inputs.create_support.connect(create_support)
        if mapping_on_scoping !=None:
            self.inputs.mapping_on_scoping.connect(mapping_on_scoping)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluates a result on specified coordinates (interpolates results inside elements with shape functions).""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "coordinates", type_names=["field","fields_container"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "create_support", type_names=["bool"], optional=True, document="""if this pin is set to true, then, a support associated to the fields consisting of points is created"""), 
                                 3 : PinSpecification(name = "mapping_on_scoping", type_names=["bool"], optional=True, document="""if this pin is set to true, then the mapping between the coordinates and the fields is created only on the first field scoping"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""if the first field in input has no mesh in support, then the mesh in this pin is expected (default is false), if a meshes container with several meshes is set, it should be on the same label spaces as the coordinates fields container""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mapping")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsOnCoordinates 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsOnCoordinates 
        """
        return super().outputs


#internal name: mapping
#scripting name: on_coordinates
class InputsOnCoordinates(_Inputs):
    """Intermediate class used to connect user inputs to on_coordinates operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mapping.on_coordinates()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_coordinates = dpf.Field()
      >>> op.inputs.coordinates.connect(my_coordinates)
      >>> my_create_support = bool()
      >>> op.inputs.create_support.connect(my_create_support)
      >>> my_mapping_on_scoping = bool()
      >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(on_coordinates._spec().inputs, op)
        self._fields_container = Input(on_coordinates._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._coordinates = Input(on_coordinates._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._coordinates)
        self._create_support = Input(on_coordinates._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._create_support)
        self._mapping_on_scoping = Input(on_coordinates._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._mapping_on_scoping)
        self._mesh = Input(on_coordinates._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def coordinates(self):
        """Allows to connect coordinates input to the operator

        Parameters
        ----------
        my_coordinates : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.coordinates.connect(my_coordinates)
        >>> #or
        >>> op.inputs.coordinates(my_coordinates)

        """
        return self._coordinates

    @property
    def create_support(self):
        """Allows to connect create_support input to the operator

        - pindoc: if this pin is set to true, then, a support associated to the fields consisting of points is created

        Parameters
        ----------
        my_create_support : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.create_support.connect(my_create_support)
        >>> #or
        >>> op.inputs.create_support(my_create_support)

        """
        return self._create_support

    @property
    def mapping_on_scoping(self):
        """Allows to connect mapping_on_scoping input to the operator

        - pindoc: if this pin is set to true, then the mapping between the coordinates and the fields is created only on the first field scoping

        Parameters
        ----------
        my_mapping_on_scoping : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.mapping_on_scoping.connect(my_mapping_on_scoping)
        >>> #or
        >>> op.inputs.mapping_on_scoping(my_mapping_on_scoping)

        """
        return self._mapping_on_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: if the first field in input has no mesh in support, then the mesh in this pin is expected (default is false), if a meshes container with several meshes is set, it should be on the same label spaces as the coordinates fields container

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsOnCoordinates(_Outputs):
    """Intermediate class used to get outputs from on_coordinates operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mapping.on_coordinates()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(on_coordinates._spec().outputs, op)
        self._fields_container = Output(on_coordinates._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mapping.on_coordinates()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
field_to_csv
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "serialization" category
"""

class field_to_csv(Operator):
    """Exports a field or a fields container into a csv file

      available inputs:
        - field_or_fields_container (FieldsContainer, Field)
        - file_path (str)
        - storage_type (int) (optional)

      available outputs:


      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.field_to_csv()

      >>> # Make input connections
      >>> my_field_or_fields_container = dpf.FieldsContainer()
      >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_storage_type = int()
      >>> op.inputs.storage_type.connect(my_storage_type)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.field_to_csv(field_or_fields_container=my_field_or_fields_container,file_path=my_file_path,storage_type=my_storage_type)

      >>> # Get output data"""
    def __init__(self, field_or_fields_container=None, file_path=None, storage_type=None, config=None, server=None):
        super().__init__(name="field_to_csv", config = config, server = server)
        self._inputs = InputsFieldToCsv(self)
        self._outputs = OutputsFieldToCsv(self)
        if field_or_fields_container !=None:
            self.inputs.field_or_fields_container.connect(field_or_fields_container)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)
        if storage_type !=None:
            self.inputs.storage_type.connect(storage_type)

    @staticmethod
    def _spec():
        spec = Specification(description="""Exports a field or a fields container into a csv file""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field_or_fields_container", type_names=["fields_container","field"], optional=False, document="""field_or_fields_container"""), 
                                 1 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "storage_type", type_names=["int32"], optional=True, document="""storage type : if matrices (without any particularity) are included in the fields container, the storage format can be chosen. 0 : flat/line format, 1 : ranked format. If 1 is chosen, the csv can not be read by "csv to field" operator anymore. Default : 0.""")},
                             map_output_pin_spec={
})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "field_to_csv")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFieldToCsv 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFieldToCsv 
        """
        return super().outputs


#internal name: field_to_csv
#scripting name: field_to_csv
class InputsFieldToCsv(_Inputs):
    """Intermediate class used to connect user inputs to field_to_csv operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.field_to_csv()
      >>> my_field_or_fields_container = dpf.FieldsContainer()
      >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_storage_type = int()
      >>> op.inputs.storage_type.connect(my_storage_type)
    """
    def __init__(self, op: Operator):
        super().__init__(field_to_csv._spec().inputs, op)
        self._field_or_fields_container = Input(field_to_csv._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field_or_fields_container)
        self._file_path = Input(field_to_csv._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._file_path)
        self._storage_type = Input(field_to_csv._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._storage_type)

    @property
    def field_or_fields_container(self):
        """Allows to connect field_or_fields_container input to the operator

        - pindoc: field_or_fields_container

        Parameters
        ----------
        my_field_or_fields_container : FieldsContainer, Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.field_or_fields_container.connect(my_field_or_fields_container)
        >>> #or
        >>> op.inputs.field_or_fields_container(my_field_or_fields_container)

        """
        return self._field_or_fields_container

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

    @property
    def storage_type(self):
        """Allows to connect storage_type input to the operator

        - pindoc: storage type : if matrices (without any particularity) are included in the fields container, the storage format can be chosen. 0 : flat/line format, 1 : ranked format. If 1 is chosen, the csv can not be read by "csv to field" operator anymore. Default : 0.

        Parameters
        ----------
        my_storage_type : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.field_to_csv()
        >>> op.inputs.storage_type.connect(my_storage_type)
        >>> #or
        >>> op.inputs.storage_type(my_storage_type)

        """
        return self._storage_type

class OutputsFieldToCsv(_Outputs):
    """Intermediate class used to get outputs from field_to_csv operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.field_to_csv()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(field_to_csv._spec().outputs, op)
        pass 




"""
min_max
=======
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class min_max(Operator):
    """Compute the component-wise minimum (out 0) and maximum (out 1) over a field.

      available inputs:
        - field (Field, FieldsContainer)

      available outputs:
        - field_min (Field)
        - field_max (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.min_max()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.min_max(field=my_field)

      >>> # Get output data
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()"""
    def __init__(self, field=None, config=None, server=None):
        super().__init__(name="min_max", config = config, server = server)
        self._inputs = InputsMinMax(self)
        self._outputs = OutputsMinMax(self)
        if field !=None:
            self.inputs.field.connect(field)

    @staticmethod
    def _spec():
        spec = Specification(description="""Compute the component-wise minimum (out 0) and maximum (out 1) over a field.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field_min", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "field_max", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "min_max")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinMax 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMinMax 
        """
        return super().outputs


#internal name: min_max
#scripting name: min_max
class InputsMinMax(_Inputs):
    """Intermediate class used to connect user inputs to min_max operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
    """
    def __init__(self, op: Operator):
        super().__init__(min_max._spec().inputs, op)
        self._field = Input(min_max._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)

    @property
    def field(self):
        """Allows to connect field input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_field : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

class OutputsMinMax(_Outputs):
    """Intermediate class used to get outputs from min_max operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field_min = op.outputs.field_min()
      >>> result_field_max = op.outputs.field_max()
    """
    def __init__(self, op: Operator):
        super().__init__(min_max._spec().outputs, op)
        self._field_min = Output(min_max._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field_min)
        self._field_max = Output(min_max._spec().output_pin(1), 1, op) 
        self._outputs.append(self._field_max)

    @property
    def field_min(self):
        """Allows to get field_min output of the operator


        Returns
        ----------
        my_field_min : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_min = op.outputs.field_min() 
        """
        return self._field_min

    @property
    def field_max(self):
        """Allows to get field_max output of the operator


        Returns
        ----------
        my_field_max : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_max = op.outputs.field_max() 
        """
        return self._field_max




"""
stl_export
==========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from meshOperatorsCore plugin, from "mesh" category
"""

class stl_export(Operator):
    """export a mesh into a stl file.

      available inputs:
        - mesh (MeshedRegion)
        - file_path (str)

      available outputs:
        - data_sources (DataSources)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.stl_export()

      >>> # Make input connections
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.stl_export(mesh=my_mesh,file_path=my_file_path)

      >>> # Get output data
      >>> result_data_sources = op.outputs.data_sources()"""
    def __init__(self, mesh=None, file_path=None, config=None, server=None):
        super().__init__(name="stl_export", config = config, server = server)
        self._inputs = InputsStlExport(self)
        self._outputs = OutputsStlExport(self)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)

    @staticmethod
    def _spec():
        spec = Specification(description="""export a mesh into a stl file.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "stl_export")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStlExport 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsStlExport 
        """
        return super().outputs


#internal name: stl_export
#scripting name: stl_export
class InputsStlExport(_Inputs):
    """Intermediate class used to connect user inputs to stl_export operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.stl_export()
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
    """
    def __init__(self, op: Operator):
        super().__init__(stl_export._spec().inputs, op)
        self._mesh = Input(stl_export._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._mesh)
        self._file_path = Input(stl_export._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._file_path)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.stl_export()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.stl_export()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

class OutputsStlExport(_Outputs):
    """Intermediate class used to get outputs from stl_export operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.stl_export()
      >>> # Connect inputs : op.inputs. ...
      >>> result_data_sources = op.outputs.data_sources()
    """
    def __init__(self, op: Operator):
        super().__init__(stl_export._spec().outputs, op)
        self._data_sources = Output(stl_export._spec().output_pin(0), 0, op) 
        self._outputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to get data_sources output of the operator


        Returns
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.stl_export()
        >>> # Connect inputs : op.inputs. ...
        >>> result_data_sources = op.outputs.data_sources() 
        """
        return self._data_sources




"""
PropertyField
=============
"""

from ansys.dpf.core.common import natures, locations
from ansys.dpf.core.field_base import _FieldBase, _LocalFieldBase


class PropertyField(_FieldBase):
    """Describes field properties such as connectivity.

    This class is a field with integer values instead of double values.

    Parameters
    ----------
    nentities: int
        Number of entities that the property field is to contain. The
        default is ``0``.
    nature: core.natures
        Nature of the property field, such as scalar or vector.
    location : str, optional
        Location of the property field. Options are ``"Nodal"`` or ``"Elemental"``.
        The default is ``"Nodal"``.
    property field : str, optional
        Name of the property field. The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Returns
    -------
    property_field: PropertyField

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> pfield = dpf.PropertyField()
    >>> list_ids = [1, 2, 4, 6, 7]
    >>> scop = dpf.Scoping(ids = list_ids, location = dpf.locations.nodal)
    >>> pfield.scoping = scop
    >>> list_data = [20, 30, 50, 70, 80]
    >>> pfield.data = list_data

    """

    def __init__(
        self,
        nentities=0,
        nature=natures.scalar,
        location=locations.nodal,
        property_field=None,
        server=None,
    ):
        super().__init__(nentities, nature, location, True, property_field, server)

    @property
    def location(self):
        """Location of the property field.

        A property field contains a scoping, which is the location that is read.
        To update location, directly update the scoping location.

        Returns
        -------
        location : str
            Location string, which is either ``"Nodal"`` or ``"Elemental"``.

        Examples
        --------
        Create a property field and request the location.

        >>> from ansys.dpf import core as dpf
        >>> pfield = dpf.PropertyField()
        >>> list_ids = [1, 2, 4, 6, 7]
        >>> scop = dpf.Scoping(ids = list_ids, location = dpf.locations.nodal)
        >>> pfield.scoping = scop
        >>> pfield.scoping.location = dpf.locations.nodal
        >>> pfield.location
        'Nodal'

        """
        if self.scoping:
            return self.scoping.location
        else:
            return None

    @location.setter
    def location(self, value):
        """Change the property field location.

        Parameters
        ----------
        location : str or locations
            Location string, which is either ``"Nodal"`` or ``"Elemental"``.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> pfield = dpf.PropertyField()
        >>> scop = dpf.Scoping(ids = list_ids, location = dpf.locations.nodal)
        >>> pfield.scoping = scop
        >>> pfield.location = 'Nodal'
        >>> pfield.location
        'Nodal'

        """
        if self.scoping:
            self.scoping.location = value
        else:
            raise Exception(
                "Property field location is based on scoping, and scoping is not defined"
            )

    def as_local_field(self):
        """Create a deep copy of the field locally.

        This copy can then be accessed and modified locally, without a request
        being sent to the server. This method should be used in a ``with``
        statement so that the local field is released and the data is sent to
        the server in one action. If it's not used in a ``with`` statement, the
        method ``release_data()`` should be used to actually update the field.

        .. warning::
           If this ``as_local_field`` method is not used as a context manager in a
           `with` statement or if the method ``release_data()`` is not called,
           the data will not actually be updated.

        Returns
        -------
        local_field : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> num_entities = 5
        >>> field_to_local = dpf.PropertyField(num_entities, dpf.natures.scalar)
        >>> with field_to_local.as_local_field() as f:
        ...     for i in range(1,num_entities+1):
        ...         f.append(list(range(i,i+3)),i)
        ...         f.get_entity_data(i-1)
        array([1, 2, 3])
        array([2, 3, 4])
        array([3, 4, 5])
        array([4, 5, 6])
        array([5, 6, 7])

        """
        return _LocalPropertyField(self)


class _LocalPropertyField(_LocalFieldBase, PropertyField):
    """Caches the internal data of a field so that it can be modified locally.

    A single update request is sent to the server when the local field is deleted.

    Parameters
    ----------
    field : PropertyField
        Property field to copy.
    """

    def __init__(self, field):
        super().__init__(field)



from .field_band_pass_fc import field_band_pass_fc 
from .scoping_low_pass import scoping_low_pass 
from .field_high_pass import field_high_pass 
from .scoping_high_pass import scoping_high_pass 
from .field_high_pass_fc import field_high_pass_fc 
from .field_low_pass import field_low_pass 
from .field_low_pass_fc import field_low_pass_fc 
from .field_band_pass import field_band_pass 
from .scoping_band_pass import scoping_band_pass 



"""
txt_file_to_dpf
===============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "utility" category
"""

class txt_file_to_dpf(Operator):
    """Take an input string and parse it into dataProcessing type.

      available inputs:
        - input_string (str)

      available outputs:
        - any_output1 ()
        - any_output2 ()

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.utility.txt_file_to_dpf()

      >>> # Make input connections
      >>> my_input_string = str()
      >>> op.inputs.input_string.connect(my_input_string)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.utility.txt_file_to_dpf(input_string=my_input_string)

      >>> # Get output data
      >>> result_any_output1 = op.outputs.any_output1()
      >>> result_any_output2 = op.outputs.any_output2()"""
    def __init__(self, input_string=None, config=None, server=None):
        super().__init__(name="text_parser", config = config, server = server)
        self._inputs = InputsTxtFileToDpf(self)
        self._outputs = OutputsTxtFileToDpf(self)
        if input_string !=None:
            self.inputs.input_string.connect(input_string)

    @staticmethod
    def _spec():
        spec = Specification(description="""Take an input string and parse it into dataProcessing type.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "input_string", type_names=["string"], optional=False, document="""ex: 'double:1.0', 'int:1', 'vector<double>:1.0;1.0'""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "any_output", type_names=[], optional=False, document="""any output"""), 
                                 1 : PinSpecification(name = "any_output", type_names=[], optional=False, document="""any output""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "text_parser")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsTxtFileToDpf 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsTxtFileToDpf 
        """
        return super().outputs


#internal name: text_parser
#scripting name: txt_file_to_dpf
class InputsTxtFileToDpf(_Inputs):
    """Intermediate class used to connect user inputs to txt_file_to_dpf operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.txt_file_to_dpf()
      >>> my_input_string = str()
      >>> op.inputs.input_string.connect(my_input_string)
    """
    def __init__(self, op: Operator):
        super().__init__(txt_file_to_dpf._spec().inputs, op)
        self._input_string = Input(txt_file_to_dpf._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._input_string)

    @property
    def input_string(self):
        """Allows to connect input_string input to the operator

        - pindoc: ex: 'double:1.0', 'int:1', 'vector<double>:1.0;1.0'

        Parameters
        ----------
        my_input_string : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.txt_file_to_dpf()
        >>> op.inputs.input_string.connect(my_input_string)
        >>> #or
        >>> op.inputs.input_string(my_input_string)

        """
        return self._input_string

class OutputsTxtFileToDpf(_Outputs):
    """Intermediate class used to get outputs from txt_file_to_dpf operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.txt_file_to_dpf()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(txt_file_to_dpf._spec().outputs, op)
        pass 




"""
mesh_scoping_factory
====================

Contains functions to simplify creating mesh scopings.
"""

from ansys.dpf.core import Scoping
from ansys.dpf.core import errors as dpf_errors
from ansys.dpf.core.common import locations


def nodal_scoping(node_ids, server=None):
    """Create a specific nodal :class:`ansys.dpf.core.Scoping` associated with a mesh.

    Parameters
    ----------
    node_ids : List of int
        List of IDs for the nodes.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Returns
    -------
    scoping : ansys.dpf.core.Scoping
    """
    if not isinstance(node_ids, list):
        raise dpf_errors.InvalidTypeError("list", "node_ids")
    scoping = Scoping(server=server, ids=node_ids, location=locations.nodal)
    return scoping


def elemental_scoping(element_ids, server=None):
    """Create a specific elemental :class:`ansys.dpf.core.Scoping` associated with a mesh.

    Parameters
    ----------
    element_ids : List of int
        List of IDs for the elements.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Returns
    -------
    scoping : ansys.dpf.core.Scoping
    """
    if not isinstance(element_ids, list):
        raise dpf_errors.InvalidTypeError("list", "element_ids")
    scoping = Scoping(server=server, ids=element_ids, location=locations.elemental)
    return scoping


def named_selection_scoping(named_selection_name, model, server=None):
    """Create a specific :class:`ansys.dpf.core.Scoping` associated with a specified model's mesh.

    Parameters
    ----------
    named_selection_name : str
        Name of the named selection.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Returns
    -------
    scoping : ansys.dpf.core.Scoping
    """
    return model.metadata.named_selection(named_selection_name)
